# [ltijs](README.md) - Platform

> Platform class


[![JavaScript Style Guide](https://img.shields.io/badge/code_style-standard-brightgreen.svg)](https://standardjs.com)
[![MIT License](https://img.shields.io/github/license/Cvmcosta/ltijs.svg)](LICENSE)


## Table of Contents

- [Introduction](#introduction)
- [Documentation](#documentation)
- [Usage](#usage)
- [License](#license)

---


## Introduction
The ```Platform``` class represents a [Lti Consumer](https://www.imsglobal.org/spec/lti/v1p3/#platforms-and-tools).


---


## Documentation



#### Platform.constructor(name, platformUrl, clientId, authenticationEndpoint, accesstokenEndpoint, kid, _ENCRYPTIONKEY, _authConfig) 





##### Parameters

| Name | Type | Description |  |
| ---- | ---- | ----------- | -------- |
| name | `String`  | Platform name. | &nbsp; |
| platformUrl | `String`  | Platform url. | &nbsp; |
| clientId | `String`  | Client Id generated by the platform. | &nbsp; |
| authenticationEndpoint | `String`  | Authentication endpoint that the tool will use to authenticate within the platform. | &nbsp; |
| accesstokenEndpoint | `String`  | Access token endpoint for the platform. | &nbsp; |
| kid | `String`  | Key id for local keypair used to sign messages to this platform. | &nbsp; |
| _ENCRYPTIONKEY | `String`  | Encryption key used | &nbsp; |
| _authConfig | `Object`  | Authentication configurations for the platform. | &nbsp; |






#### async Platform.platformName([name]) 

Sets/Gets the platform name.




##### Parameters

| Name | Type | Description |  |
| ---- | ---- | ----------- | -------- |
| name | `String`  | Platform name. | *Optional* |




##### Returns


- Promise that resolves `true` or the `platform name` if it succeeds and `false` if it fails.



#### async Platform.platformUrl([url]) 

Sets/Gets the platform url.




##### Parameters

| Name | Type | Description |  |
| ---- | ---- | ----------- | -------- |
| url | `String`  | Platform url. | *Optional* |




##### Returns


- Promise that resolves `true` or the `platform url` if it succeeds and `false` if it fails.




#### async Platform.platformClientId([clientId]) 

Sets/Gets the platform client id.




##### Parameters

| Name | Type | Description |  |
| ---- | ---- | ----------- | -------- |
| clientId | `String`  | Platform client id. | *Optional* |




##### Returns


- Promise that resolves `true` or the `platform name` if it succeeds and `false` if it fails.




#### Platform.platformKid() 

Gets the platform key_id.






##### Returns


- `String` representing the platform keys id.



#### async  Platform.platformPublicKey() 

Gets the RSA public key assigned to the platform.






##### Returns


- Promise that resolves the `platform public key` if it succeeds and `false` if it fails.




#### async Platform.platformPrivateKey() 

Gets the RSA private key assigned to the platform.






##### Returns

- Promise that resolves the `platform private key` if it succeeds and `false` if it fails.



#### async Platform.platformAuthConfig(method, key) 

Sets/Gets the platform authorization configurations used to validate it's messages.




##### Parameters

| Name | Type | Description |  |
| ---- | ---- | ----------- | -------- |
| method | `String`  | Method of authorization "RSA_KEY" or "JWK_KEY" or "JWK_SET". | &nbsp; |
| key | `String`  | Either the RSA public key provided by the platform, or the JWK key, or the JWK keyset address. | &nbsp; |




##### Returns



- Promise that resolves `true` or the `platform authentication configuration` if it succeeds and `false` if it fails.




#### async Platform.platformAuthEndpoint([authEndpoint]) 

Sets/Gets the platform authorization endpoint used to perform the OIDC login.




##### Parameters

| Name | Type | Description |  |
| ---- | ---- | ----------- | -------- |
| authEndpoint | `String`  | Platform authorization endpoint. | *Optional* |




##### Returns


- Promise that resolves `true` or the `platform authentication endpoint` if it succeeds and `false` if it fails.




#### async Platform.platformAccessTokenEndpoint([accesstokenEndpoint]) 

Sets/Gets the platform access token endpoint used to authenticate messages to the platform.




##### Parameters

| Name | Type | Description |  |
| ---- | ---- | ----------- | -------- |
| accesstokenEndpoint | `String`  | Platform access token endpoint. | *Optional* |




##### Returns



- Promise that resolves `true` or the `platform access token endpoint` if it succeeds and `false` if it fails.




#### async Platform.platformAccessToken() 

Gets the platform access token or attempts to generate a new one.






##### Returns


- Promise that resolves the `platform access token endpoint` if it succeeds and `false` if it fails.




#### async Platform.remove() 

Deletes a registered platform.






##### Returns

- Promise that resolves `true` if it succeeds and `false` if it fails.


---

## Usage

### Registering platform

The [Lti Provider](provider.md) method `registerPlatform()` returns a Promise that resolves the created `Platform` or `false` if some error occurs.

```javascript
let plat = await lti.registerPlatform(
  'http://platform/url', 
  'Platform Name', 'ClientIdThePlatformCreatedForYourApp', 
  'http://platform/AuthorizationUrl', 
  'http://platform/AccessTokenUrl', 
  { method: 'JWK_SET', key: 'http://platform/keyset' }
  )
```



---

## License

[![MIT License](https://img.shields.io/github/license/Cvmcosta/ltijs.svg)](LICENSE)

- **[MIT license](http://opensource.org/licenses/mit-license.php)**
